// BonSrcDecoder.cpp: CBonSrcDecoder NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BonSrcDecoder.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#pragma comment(lib, "BonDriver.lib")


//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CBonSrcDecoder::CBonSrcDecoder(IEventHandler *pEventHandler)
	: CMediaDecoder(pEventHandler, 0UL, 1UL)
	, m_pBonDriver(NULL)
	, m_hStreamRecvThread(NULL)
	, m_TsStream(0x10000UL)
	, m_bKillSignal(true)
	, m_bIsPlaying(false)
	, m_dwLastError(BSDEC_NOERROR)
{

}

CBonSrcDecoder::~CBonSrcDecoder()
{
	CloseTuner();
}

void CBonSrcDecoder::Reset(void)
{
	CBlockLock Lock(&m_CriticalLock);

	// ̃Xg[j
	if(m_bIsPlaying){
		// ̃Xg[j
		m_pBonDriver->PurgeTsStream();
		}

	// ʃfR[_Zbg
	CMediaDecoder::Reset();
}

const bool CBonSrcDecoder::InputMedia(CMediaData *pMediaData, const DWORD dwInputIndex)
{
	// \[XfR[_̂ߓ͂͏Ȃ
	return false;
}

const bool CBonSrcDecoder::OpenTuner(void)
{
	CBlockLock Lock(&m_CriticalLock);
	
	// I[v`FbN
	if(m_pBonDriver){
		m_dwLastError = BSDEC_ALREADYOPEN;
		return false;
		}

	// hCoCX^X
	if(!(m_pBonDriver = ::CreateBonDriver())){
		m_dwLastError = BSDEC_TUNERERROR;
		return false;
		}

	// `[iJ
	if(!m_pBonDriver->OpenTuner()){
		CloseTuner();
		m_dwLastError = BSDEC_TUNERERROR;
		return false;
		}

	// `lZbg
	if(!m_pBonDriver->SetChannel(13U)){
		CloseTuner();
		m_dwLastError = BSDEC_TUNERERROR;
		return false;
		}

	// Xg[MXbhN
	DWORD dwThreadID = 0UL;
	m_bKillSignal = false;
	m_bIsPlaying = false;

	if(!(m_hStreamRecvThread = ::CreateThread(NULL, 0UL, CBonSrcDecoder::StreamRecvThread, (LPVOID)this, 0UL, &dwThreadID))){
		CloseTuner();
		m_dwLastError = BSDEC_INTERNALERROR;
		return false;
		}

	m_dwLastError = BSDEC_NOERROR;

	return true;
}

const bool CBonSrcDecoder::CloseTuner(void)
{
	CBlockLock Lock(&m_CriticalLock);

	// Xg[~
	m_bIsPlaying = false;
	m_bKillSignal = true;

	if(m_hStreamRecvThread){
		// Xg[MXbh~
		if(::WaitForSingleObject(m_hStreamRecvThread, 1000UL) != WAIT_OBJECT_0){
			// XbhI
			::TerminateThread(m_hStreamRecvThread, 0UL);
			}

		::CloseHandle(m_hStreamRecvThread);
		m_hStreamRecvThread = NULL;
		}

	if(m_pBonDriver){
		// `[i
		m_pBonDriver->CloseTuner();

		// hCoCX^XJ
		m_pBonDriver->Release();
		m_pBonDriver = NULL;		
		}

	m_dwLastError = BSDEC_NOERROR;

	return true;
}

const bool CBonSrcDecoder::Play(void)
{
	CBlockLock Lock(&m_CriticalLock);
	
	if(!m_pBonDriver){
		// `[iJĂȂ
		m_dwLastError = BSDEC_TUNERNOTOPEN;
		return false;
		}

	if(m_bIsPlaying){
		// ɍĐ
		m_dwLastError = BSDEC_ALREADYPLAYING;
		return false;
		}

	// ̃Xg[j
	m_pBonDriver->PurgeTsStream();

	// ʃfR[_Zbg
	CMediaDecoder::Reset();

	// Xg[ĐԂɂ
	m_bIsPlaying = true;
	m_dwLastError = BSDEC_NOERROR;

	return true;
}

const bool CBonSrcDecoder::Stop(void)
{
	if(!m_bIsPlaying){
		// Xg[͍ĐłȂ
		m_dwLastError = BSDEC_NOTPLAYING;
		return false;
		}

	m_bIsPlaying = false;
	m_dwLastError = BSDEC_NOERROR;

	return true;
}

const bool CBonSrcDecoder::SetChannel(const BYTE byChannel)
{
	CBlockLock Lock(&m_CriticalLock);

	if(!m_pBonDriver){
		// `[iJĂȂ
		m_dwLastError = BSDEC_TUNERNOTOPEN;
		return false;
		}

	// ̃Xg[j
	m_pBonDriver->PurgeTsStream();

	// `lύX
	if(!m_pBonDriver->SetChannel(byChannel)){
		m_dwLastError = BSDEC_TUNERERROR;
		return false;
		}

	// ʃfR[_Zbg
	CMediaDecoder::Reset();

	m_dwLastError = BSDEC_NOERROR;
	
	return true;
}

const float CBonSrcDecoder::GetSignalLevel(void)
{
	if(!m_pBonDriver){
		// `[iJĂȂ
		m_dwLastError = BSDEC_TUNERNOTOPEN;
		return 0.0f;
		}

	m_dwLastError = BSDEC_NOERROR;

	// MxԂ
	return m_pBonDriver->GetSignalLevel();
}

const DWORD CBonSrcDecoder::GetLastError(void) const
{
	// ŌɔG[Ԃ
	return m_dwLastError;
}

const bool CBonSrcDecoder::PurgeStream(void)
{
	CBlockLock Lock(&m_CriticalLock);

	if(!m_pBonDriver){
		// `[iJĂȂ
		m_dwLastError = BSDEC_TUNERNOTOPEN;
		return false;
		}

	// ̃Xg[j
	m_pBonDriver->PurgeTsStream();

	// ʃfR[_Zbg
	CMediaDecoder::Reset();
	
	m_dwLastError = BSDEC_NOERROR;

	return true;
}

void CBonSrcDecoder::OnTsStream(BYTE *pStreamData, DWORD dwStreamSize)
{
	CBlockLock Lock(&m_CriticalLock);

	if(!m_bIsPlaying)return;

	// ŏʃfR[_ɓ͂
	m_TsStream.SetData(pStreamData, dwStreamSize);
	OutputMedia(&m_TsStream);
}

DWORD WINAPI CBonSrcDecoder::StreamRecvThread(LPVOID pParam)
{
	CBonSrcDecoder *pThis = static_cast<CBonSrcDecoder *>(pParam);

	BYTE *pStreamData = NULL;
	DWORD dwStreamSize = 0UL;
	DWORD dwStreamRemain = 0UL;

	// `[iTSf[^oXbh
	while(!pThis->m_bKillSignal){
		
		// ȗ̂߃|[O̗p
		do{
			if(pThis->m_pBonDriver->GetTsStream(&pStreamData, &dwStreamSize, &dwStreamRemain)){
				if(pStreamData && dwStreamSize){
					pThis->OnTsStream(pStreamData, dwStreamSize);
					}
				}
			}
		while(dwStreamRemain);

		// EFCg(24MbpsƂĎ̃f[^܂Ŗ15ms)
		::Sleep(1UL);
		}

	return 0UL;
}

